#!/usr/bin/env python

import csv
import sys
import argparse
from collections import defaultdict
from typing import Set, Optional, Dict, List, Tuple, Any, Iterable, Iterator

Args = argparse.Namespace

def as_set(s : str) -> Set[str]:
    if not s.strip():
        return set()

    return {elm.strip() for elm in s.split(',')}

def as_str(s : Set[str]) -> str:
    return ','.join(sorted(s))

def load_subjects(csv_in : Any) -> Dict[str, List[Tuple[int, Set[str], Set[str]]]]:
    subjects : Dict[str, List[Tuple[int, Set[str], Set[str]]]] = defaultdict(list)

    for position_s, subj, menu_s, choice_s in csv_in:
        subjects[subj].append((int(position_s), as_set(menu_s), as_set(choice_s)))

    return subjects

# needed on Windows
def crlf_normaliser(io : Iterable[str]) -> Iterator[str]:
    for line in io:
        yield line.strip()

def main(args : Args) -> None:
    csv_in = csv.reader(crlf_normaliser(sys.stdin))
    csv_out = csv.writer(sys.stdout)

    header = next(csv_in)
    assert header == ['position', 'subject', 'menu', 'choice']
    csv_out.writerow(header)

    for subject, rows_raw in load_subjects(csv_in).items():
        rows_sorted = sorted(rows_raw, key=lambda r: r[0])

        if (args.first and not args.relaxed) or args.last:
            # make sure that the numbering is consistent
            # we allow this for --first
            # but for --last, we really rely on this and we can't relax it
            #
            # position is 1-based
            assert [r[0]-1 for r in rows_sorted] == list(range(len(rows_sorted))), \
                f'inconsistent numbering: ' + str([r[0] for r in rows_sorted])

        for position, menu, choice in rows_sorted:
            hit : Optional[bool] = None

            if args.first is not None:
                assert hit is None
                # position is 1-based
                hit = position <= args.first

            if args.last is not None:
                assert hit is None
                # position is 1-based
                hit = (position >= len(rows_sorted)-args.last+1)

            if args.menu is not None:
                assert hit is None
                hit = as_set(args.menu) == menu

            if args.deferrals:
                assert hit is None
                hit = not choice

            assert hit is not None

            if args.action == 'drop':
                keep = not hit
            elif args.action == 'keep':
                keep = hit
            else:
                raise Exception('unknown action: ' + args.action)

            if keep:
                csv_out.writerow((position, subject, as_str(menu), as_str(choice)))

if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument('action', choices=('drop', 'keep'))
    ap.add_argument('--relaxed', action='store_true', default=False)
    ap.add_argument('-f', '--first', type=int)
    ap.add_argument('-l', '--last', type=int)
    ap.add_argument('-d', '--deferrals', default=False, action='store_true')
    ap.add_argument('-m', '--menu')
    main(ap.parse_args())
